using System;
using System.Text.RegularExpressions;
using ImTryin.StateMachineDesigner.DomainModel;
using Microsoft.VisualStudio.Modeling;

namespace ImTryin.StateMachineDesigner.Designer
{
    [RuleOn( typeof ( Event ), FireTime = TimeToFire.TopLevelCommit )]
    public sealed class DomainModelEventChangeRule : ChangeRule
    {
        public override void ElementAttributeChanged( ElementAttributeChangedEventArgs e )
        {
            Event oEvent = (Event) e.ModelElement;

            if ( ( e.MetaAttribute.Id == Event.ReturnMetaAttributeGuid ) ||
                 ( e.MetaAttribute.Id == Event.AccessMetaAttributeGuid ) ||
                 ( e.MetaAttribute.Id == Event.ParametersMetaAttributeGuid ) )
            {
                string notation = oEvent.Return + " " + oEvent.Access + "(" + oEvent.Parameters + ")";
                if ( notation != oEvent.Notation )
                    oEvent.Notation = notation;

                if ( e.MetaAttribute.Id == Event.AccessMetaAttributeGuid )
                {
                    foreach ( Transition transition in oEvent.Transitions )
                    {
                        if ( transition.Trigger != oEvent.Access )
                            transition.Trigger = oEvent.Access;
                    }
                }
            }
            else if ( e.MetaAttribute.Id == Event.NotationMetaAttributeGuid )
            {
                string notation = oEvent.Notation.Trim();

                string sReturn = "";
                string access = notation;
                string parameters = "";

                int lastOpenParenthesis = notation.LastIndexOf( "(" );
                if ( ( -1 != lastOpenParenthesis ) && ( notation.EndsWith( ")" ) ) )
                {
                    access = notation.Substring( 0, lastOpenParenthesis );
                    parameters =
                        notation.Substring( lastOpenParenthesis + 1, notation.Length - lastOpenParenthesis - 2 );
                }

                int firstSpace = access.IndexOf( " " );
                if ( ( -1 != firstSpace ) &&
                     ( Regex.Match( access.Substring( 0, firstSpace ), @"[A-Za-z_0-9\.]*" ).Success ) )
                {
                    sReturn = access.Substring( 0, firstSpace );
                    access = access.Substring( firstSpace + 1 );
                }

                if ( sReturn != oEvent.Return )
                    oEvent.Return = sReturn;
                if ( access != oEvent.Access )
                    oEvent.Access = access;
                if ( parameters != oEvent.Parameters )
                    oEvent.Parameters = parameters;
            }
        }
    }

    [RuleOn( typeof ( EventHasTransitions ), FireTime = TimeToFire.TopLevelCommit )]
    public sealed class DomainModelEventHasTransitionsRemovingRule : RemovingRule
    {
        public override void ElementRemoving( ElementRemovingEventArgs e )
        {
            EventHasTransitions eventHasTransitions = (EventHasTransitions) e.ModelElement;
            if ( eventHasTransitions.Event.IsRemoving && !eventHasTransitions.Event.CodeGeneration.IsRemoving )
            {
                foreach ( Transition transition in eventHasTransitions.Event.Transitions )
                {
                    if ( !transition.IsRemoving )
                        throw new InvalidOperationException( "Cannot Remove referenced Event!" );
                }
            }

            //if ( 1 == eventHasTransitions.Event.Transitions.Count )
            //{
            //    if ( DialogResult.Yes ==
            //         MessageBox.Show( "Remove Event: " + eventHasTransitions.Event.Notation + "?", "Code Generation",
            //                          MessageBoxButtons.YesNo ) )
            //        eventHasTransitions.Event.Remove();
            //}
        }
    }


    [RuleOn( typeof ( Condition ), FireTime = TimeToFire.TopLevelCommit )]
    public sealed class DomainModelConditionChangeRule : ChangeRule
    {
        public override void ElementAttributeChanged( ElementAttributeChangedEventArgs e )
        {
            Condition condition = (Condition) e.ModelElement;

            if ( e.MetaAttribute.Id == Condition.NotationMetaAttributeGuid )
            {
                foreach ( Transition transition in condition.Transitions )
                {
                    if ( transition.Guard != condition.Notation )
                        transition.Guard = condition.Notation;
                }
            }
        }
    }

    [RuleOn( typeof ( ConditionHasTransitions ), FireTime = TimeToFire.TopLevelCommit )]
    public sealed class DomainModelConditionHasTransitionsRemovingRule : RemovingRule
    {
        public override void ElementRemoving( ElementRemovingEventArgs e )
        {
            ConditionHasTransitions conditionHasTransitions = (ConditionHasTransitions) e.ModelElement;
            if ( conditionHasTransitions.Condition.IsRemoving &&
                 !conditionHasTransitions.Condition.CodeGeneration.IsRemoving )
            {
                foreach ( Transition transition in conditionHasTransitions.Condition.Transitions )
                {
                    if ( !transition.IsRemoving )
                        throw new InvalidOperationException( "Cannot Remove referenced Condition!" );
                }
            }

            //if ( 1 == conditionHasTransitions.Condition.Transitions.Count )
            //{
            //    if ( DialogResult.Yes ==
            //         MessageBox.Show( "Remove Condition: " + conditionHasTransitions.Condition.Notation + "?",
            //                          "Code Generation", MessageBoxButtons.YesNo ) )
            //        conditionHasTransitions.Condition.Remove();
            //}
        }
    }


    [RuleOn( typeof ( Statement ), FireTime = TimeToFire.TopLevelCommit )]
    public sealed class DomainModelStatementChangeRule : ChangeRule
    {
        public override void ElementAttributeChanged( ElementAttributeChangedEventArgs e )
        {
            Statement statement = (Statement) e.ModelElement;

            if ( e.MetaAttribute.Id == Statement.NotationMetaAttributeGuid )
            {
                foreach ( Transition transition in statement.Transitions )
                {
                    if ( transition.Activity != statement.Notation )
                        transition.Activity = statement.Notation;
                }

                foreach ( StateActivity stateActivity in statement.StateActivities )
                {
                    if ( stateActivity.Activity != statement.Notation )
                        stateActivity.Activity = statement.Notation;
                }
            }
        }
    }

    [RuleOn( typeof ( StatementHasTransitions ), FireTime = TimeToFire.TopLevelCommit )]
    public sealed class DomainModelStatementHasTransitionsRemovingRule : RemovingRule
    {
        public override void ElementRemoving( ElementRemovingEventArgs e )
        {
            StatementHasTransitions statementHasTransitions = (StatementHasTransitions) e.ModelElement;
            if ( statementHasTransitions.Statement.IsRemoving &&
                 !statementHasTransitions.Statement.CodeGeneration.IsRemoving )
            {
                foreach ( Transition transition in statementHasTransitions.Statement.Transitions )
                {
                    if ( !transition.IsRemoving )
                        throw new InvalidOperationException( "Cannot Remove referenced Statement!" );
                }
                foreach ( StateActivity stateActivity in statementHasTransitions.Statement.StateActivities )
                {
                    if ( !stateActivity.IsRemoving )
                        throw new InvalidOperationException( "Cannot Remove referenced Statement!" );
                }
            }

            //if ( ( 1 == statementHasTransitions.Statement.Transitions.Count ) &&
            //     ( 0 == statementHasTransitions.Statement.StateActivities.Count ) )
            //{
            //    if ( DialogResult.Yes ==
            //         MessageBox.Show( "Remove Statement: " + statementHasTransitions.Statement.Notation + "?",
            //                          "Code Generation", MessageBoxButtons.YesNo ) )
            //        statementHasTransitions.Statement.Remove();
            //}
        }
    }

    [RuleOn( typeof ( StatementHasStateActivities ), FireTime = TimeToFire.TopLevelCommit )]
    public sealed class DomainModelStatementHasStateActivitiesRemovingRule : RemovingRule
    {
        public override void ElementRemoving( ElementRemovingEventArgs e )
        {
            StatementHasStateActivities statementHasStateActivities = (StatementHasStateActivities) e.ModelElement;
            if ( statementHasStateActivities.Statement.IsRemoving &&
                 !statementHasStateActivities.Statement.CodeGeneration.IsRemoving )
            {
                foreach ( Transition transition in statementHasStateActivities.Statement.Transitions )
                {
                    if ( !transition.IsRemoving )
                        throw new InvalidOperationException( "Cannot Remove referenced Statement!" );
                }
                foreach ( StateActivity stateActivity in statementHasStateActivities.Statement.StateActivities )
                {
                    if ( !stateActivity.IsRemoving )
                        throw new InvalidOperationException( "Cannot Remove referenced Statement!" );
                }
            }

            //if ( ( 1 == statementHasStateActivities.Statement.StateActivities.Count ) &&
            //     ( 0 == statementHasStateActivities.Statement.Transitions.Count ) )
            //{
            //    if ( DialogResult.Yes ==
            //         MessageBox.Show( "Remove Statement: " + statementHasStateActivities.Statement.Notation + "?",
            //                          "Code Generation", MessageBoxButtons.YesNo ) )
            //        statementHasStateActivities.Statement.Remove();
            //}
        }
    }


    internal static partial class GeneratedMetaModelTypes
    {
        internal static Type DomainModelEventChangeRuleType =
            typeof ( DomainModelEventChangeRule );

        internal static Type DomainModelEventHasTransitionsRemovingRuleType =
            typeof ( DomainModelEventHasTransitionsRemovingRule );


        internal static Type DomainModelConditionChangeRuleType =
            typeof ( DomainModelConditionChangeRule );

        internal static Type DomainModelConditionHasTransitionsRemovingRuleType =
            typeof ( DomainModelConditionHasTransitionsRemovingRule );


        internal static Type DomainModelStatementChangeRuleType =
            typeof ( DomainModelStatementChangeRule );

        internal static Type DomainModelStatementHasTransitionsRemovingRuleType =
            typeof ( DomainModelStatementHasTransitionsRemovingRule );

        internal static Type DomainModelStatementHasStateActivitiesRemovingRuleType =
            typeof ( DomainModelStatementHasStateActivitiesRemovingRule );
    }
}