using System.Collections.Generic;
using System.Diagnostics;
using Microsoft.VisualStudio.Modeling.Validation;

namespace ImTryin.StateMachineDesigner.DomainModel
{
    [ValidationState( ValidationState.Enabled )]
    public partial class StateMachine
    {
        [ValidationMethod( ValidationCategory.Menu | ValidationCategory.Open | ValidationCategory.Save )]
        private void ValidateStateMachine( ValidationContext context )
        {
            List< StartState > startStates = new List< StartState >();
            List< FinalState > finalStates = new List< FinalState >();
            Dictionary< string, State > states = new Dictionary< string, State >();

            foreach ( BaseState baseState in States )
            {
                if ( baseState is StartState )
                    startStates.Add( (StartState) baseState );
                else if ( baseState is FinalState )
                    finalStates.Add( (FinalState) baseState );
                else if ( baseState is State )
                {
                    State state = (State) baseState;
                    if ( !states.ContainsKey( state.Name ) )
                        states.Add( state.Name, state );
                    else
                        context.LogError( "States have the same names", "SameStateName", states[ state.Name ], state );
                }
                else
                {
                    Debug.Fail( "Unknown StateMachineElement!" );
                    context.LogError( "Unknown StateMachineElement", "UnknownStateMachineElement", baseState );
                }
            }

            if ( 1 != startStates.Count )
            {
                if ( 0 == startStates.Count )
                    context.LogError( "There is no Start State within State Machine", "NoStartState", this );
                else
                {
                    context.LogError( "There are more than one Start States within State Machine", "ManyStartState",
                                      startStates.ToArray() );
                }
            }
            else
            {
                LinksEvaluator linksEvaluator = new LinksEvaluator( startStates[ 0 ], finalStates );

                foreach ( BaseState baseState in States )
                {
                    if ( baseState is State )
                    {
                        if ( !linksEvaluator.CheckStateLinkedFromStart( baseState ) )
                            context.LogError( "State isn't accessible from Start State", "NoWayFromStart", baseState );
                        if ( ( 0 < finalStates.Count ) && ( !linksEvaluator.CheckStateLinkedToFinal( baseState ) ) )
                            context.LogWarning( "State hasn't transition way to Final State", "NoWayToFinal", baseState );
                    }
                }
            }

            if ( 0 == finalStates.Count )
                context.LogWarning( "There is no Final State within State Machine", "NoFinalState", this );
        }

        private class LinksEvaluator
        {
            private Dictionary< BaseState, BaseState > linkedFromStart =
                new Dictionary< BaseState, BaseState >();

            private Dictionary< BaseState, BaseState > linkedToFinal =
                new Dictionary< BaseState, BaseState >();

            public LinksEvaluator( StartState startState, List< FinalState > finalStates )
            {
                RecursiveFromStart( startState );

                foreach ( FinalState finalState in finalStates )
                    RecursiveFromFinal( finalState );
            }

            private void RecursiveFromStart( BaseState baseState )
            {
                linkedFromStart.Add( baseState, baseState );

                foreach ( BaseState toState in baseState.TransitionTo )
                {
                    if ( !linkedFromStart.ContainsKey( toState ) )
                        RecursiveFromStart( toState );
                }
            }

            private void RecursiveFromFinal( BaseState baseState )
            {
                linkedToFinal.Add( baseState, baseState );

                foreach ( BaseState fromState in baseState.TransitionFrom )
                {
                    if ( !linkedToFinal.ContainsKey( fromState ) )
                        RecursiveFromFinal( fromState );
                }
            }

            public bool CheckStateLinkedFromStart( BaseState baseState )
            {
                return linkedFromStart.ContainsKey( baseState );
            }

            public bool CheckStateLinkedToFinal( BaseState baseState )
            {
                return linkedToFinal.ContainsKey( baseState );
            }
        }
    }
}